package com.yizhi.course.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.exception.BizException;
import com.yizhi.course.application.domain.TestSubject;
import com.yizhi.course.application.domain.TestSubjectOption;
import com.yizhi.course.application.enums.ExceptionInfo;
import com.yizhi.course.application.mapper.TestSubjectMapper;
import com.yizhi.course.application.service.ITestService;
import com.yizhi.course.application.service.ITestSubjectOptionService;
import com.yizhi.course.application.service.ITestSubjectService;
import com.yizhi.course.application.vo.TestInfoVO;
import com.yizhi.course.application.vo.TestSubjectOptionVO;
import com.yizhi.course.application.vo.TestSubjectVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 测验试题 服务实现类
 * </p>
 *
 * @author 谢海军123
 * @since 2018-10-11
 */
@Service
public class TestSubjectServiceImpl extends ServiceImpl<TestSubjectMapper, TestSubject> implements ITestSubjectService {

    @Autowired
    private ITestService testService;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ITestSubjectOptionService optionService;

    private Logger logger = LoggerFactory.getLogger(TestSubjectServiceImpl.class);

    /**
     * 根据测试ID查询试题的总分
     *
     * @param testId
     * @return
     */
    @Override
    public Integer sumScore(Long testId) {
        return this.baseMapper.sumScore(testId);
    }

    @Override
    @Transactional
    public boolean deleteAndSort(Long id) {
        TestSubject testSubject = this.selectById(id);
        if(testSubject != null){
            this.deleteById(id);
            // 删除答案
            TestSubjectOption option = new TestSubjectOption();
            option.setTestSubjectId(id);
            optionService.delete(QueryUtil.condition(option));
            // 更新sort值
            this.baseMapper.updateSort(testSubject.getTestId(),testSubject.getSort());
        }
        return true;
    }

    /**
     * 保存测验试题
     * @param vo
     * @return
     */
    @Override
    @Transactional
    public TestSubjectVO saveTestSubject(TestSubjectVO vo){

        // 上下文
        RequestContext requestContext = ContextHolder.get();

        // 试题选项
        List<TestSubjectOptionVO> optionVOList =  vo.getOptionVOS();
        if(CollectionUtils.isEmpty(optionVOList)){
            throw new BizException(ExceptionInfo.NOT_EXIST_TEST_ANSWER.getCode(),ExceptionInfo.NOT_EXIST_TEST_ANSWER.getName());
        }

        // 判断是试题更新还是添加
        Long id = vo.getId();//试题的id
        if(id == null){//新增试题
            // 获取测试id
            Long testId = vo.getTestId();
            if(testId == null){
                // 新增一个空的测试信息
                TestInfoVO testInfoVO = new TestInfoVO();
                testInfoVO.setType(TestServiceImpl.PROCESS_TYPE);
                testInfoVO.setStatus(0);
                testInfoVO = testService.saveTest(testInfoVO);
                testId = testInfoVO.getId();
                vo.setTestId(testId);
            }
            id = idGenerator.generate();
            // 生成测验试题的序号
            Integer sort = this.baseMapper.maxSort(testId);
            vo.setSort(sort==null?0:sort.intValue());// 序号默认从1开始
            TestSubject saveObj = new TestSubject();
            vo.setId(id);
            vo.setScore(10);// 默认十分

            BeanUtils.copyProperties(vo,saveObj);
            saveObj.setCreateTime(new Date());
            saveObj.setCreateById(requestContext.getAccountId());
            saveObj.setCreateByName(requestContext.getAccountName());
            this.insert(saveObj);
        }else{
            TestSubject updateObj = new TestSubject();
            BeanUtils.copyProperties(vo,updateObj);
            updateObj.setUpdateTime(new Date());
            updateObj.setUpdateById(requestContext.getAccountId());
            updateObj.setUpdateByName(requestContext.getAccountName());
            this.updateById(updateObj);
            // 删除已经存在的答案选项
            TestSubjectOption delObj = new TestSubjectOption();
            delObj.setTestSubjectId(id);
            optionService.delete(QueryUtil.condition(delObj));
        }
        // 插入对象
        List<TestSubjectOption> listUp = new ArrayList<>();
        for(TestSubjectOptionVO obj : optionVOList){
            obj.setTestSubjectId(id);
            obj.setId(idGenerator.generate());
            TestSubjectOption optionObj = new TestSubjectOption();
            BeanUtils.copyProperties(obj,optionObj);
            listUp.add(optionObj);
        }
        optionService.insertBatch(listUp);

        return vo;
    }

    @Override
    @Transactional
    public TestSubjectVO sort(Long id, String moveType) {

        logger.info("移动的测验试题id={},移动类型={}",id,moveType);
        if (!StringUtils.equals(moveType, "up") && !StringUtils.equals(moveType, "down")) {
            throw new BizException(ExceptionInfo.MOVE_TYPE_EROOR.getCode(), ExceptionInfo.MOVE_TYPE_EROOR.getName());
        }

        TestSubject obj = this.selectById(id);
        logger.info("移动的测验试题id={},查询的数据={}",id, JSON.toJSONString(obj));
        if (obj != null) {
            int currSort = obj.getSort();
            int moveSort = 0;
            if (StringUtils.equals(moveType, "up")) {
                if (currSort <= 1) {
                    throw new BizException(ExceptionInfo.MOVE_TOP.getCode(), ExceptionInfo.MOVE_TOP.getName());
                }
                moveSort = currSort-1;
            } else if (StringUtils.equals(moveType, "down")) {
                // 获取最大的排序
                int maxSort = this.baseMapper.maxSort(obj.getTestId());
                if(currSort>=maxSort){
                    throw new BizException(ExceptionInfo.MOVE_FOOT.getCode(), ExceptionInfo.MOVE_FOOT.getName());
                }
                moveSort = currSort+1;
            }

            logger.info("currSort={},moveSort={}",currSort,moveType);
            // 更新要替换
            this.baseMapper.updateSortByTestId(obj.getTestId(),moveSort,currSort);

            // 更新移动的测验试题的sort
            //this.baseMapper.updateSortByTestId(obj.getTestId(),moveSort,currSort);
            obj.setSort(moveSort);
            this.baseMapper.updateSortById(obj.getId(),moveSort);
        }


        TestSubjectVO rt = new TestSubjectVO();
        BeanUtils.copyProperties(obj,rt);

        return rt;
    }
}
